@@ -77,16 +77,12 @@ typedef struct {
   GTlsAuthenticationMode auth_mode;
   gboolean rehandshake;
   GTlsCertificateFlags accept_flags;
-  GError *expected_client_close_error;
   GError *read_error;
-  GError *expected_server_error;
   GError *server_error;
   gboolean server_should_close;
   gboolean server_running;
   GTlsCertificate *server_certificate;
-#if GLIB_CHECK_VERSION(2, 60, 0)
   const gchar * const *server_protocols;
-#endif
 
   char buf[128];
   gssize nread, nwrote;
@@ -115,6 +111,12 @@ setup_connection (TestConnection *test, gconstpointer data)
       g_assert (!(var));                                     \
     }
 
+static void
+wait_until_server_finished (TestConnection *test)
+{
+    WAIT_UNTIL_UNSET (test->server_running);
+}
+
 static void
 teardown_connection (TestConnection *test, gconstpointer data)
 {
@@ -161,10 +163,8 @@ teardown_connection (TestConnection *test, gconstpointer data)
 
   g_main_loop_unref (test->loop);
 
-  g_clear_error (&test->expected_client_close_error);
   g_clear_error (&test->read_error);
   g_clear_error (&test->server_error);
-  g_clear_error (&test->expected_server_error);
 }
 
 static void
@@ -233,17 +233,11 @@ on_server_close_finish (GObject        *object,
                         gpointer        user_data)
 {
   TestConnection *test = user_data;
-  GError *expected_error = test->expected_server_error;
   GError *error = NULL;
 
   g_io_stream_close_finish (G_IO_STREAM (object), res, &error);
   g_assert_no_error (error);
 
-  if (expected_error)
-    g_assert_error (test->server_error, expected_error->domain, expected_error->code);
-  else
-    g_assert_no_error (test->server_error);
-
   test->server_running = FALSE;
 }
 
@@ -308,13 +302,11 @@ on_incoming_connection (GSocketService     *service,
   if (test->database)
     g_tls_connection_set_database (G_TLS_CONNECTION (test->server_connection), test->database);
 
-#if GLIB_CHECK_VERSION(2, 60, 0)
   if (test->server_protocols)
     {
       g_tls_connection_set_advertised_protocols (G_TLS_CONNECTION (test->server_connection),
                                                  test->server_protocols);
     }
-#endif
 
   stream = g_io_stream_get_output_stream (test->server_connection);
 
@@ -459,20 +451,7 @@ on_client_connection_close_finish (GObject        *object,
   GError *error = NULL;
 
   g_io_stream_close_finish (G_IO_STREAM (object), res, &error);
-
-  if (test->expected_client_close_error)
-    {
-      /* Although very rare, it's OK for broken pipe errors to not occur here if
-       * they have already occured earlier during a read. If so, there should be
-       * no error here at all.
-       */
-      if (error || !g_error_matches (test->expected_client_close_error, G_IO_ERROR, G_IO_ERROR_BROKEN_PIPE))
-        g_assert_error (error, test->expected_client_close_error->domain, test->expected_client_close_error->code);
-    }
-  else
-    {
-      g_assert_no_error (error);
-    }
+  g_assert_no_error (error);
 
   g_main_loop_quit (test->loop);
 }
@@ -533,6 +512,7 @@ test_basic_connection (TestConnection *test,
 
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   g_assert_no_error (test->read_error);
   g_assert_no_error (test->server_error);
@@ -563,6 +543,7 @@ test_verified_connection (TestConnection *test,
 
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   g_assert_no_error (test->read_error);
   g_assert_no_error (test->server_error);
@@ -932,16 +913,16 @@ test_invalid_chain_with_alternative_ca_cert (TestConnection *test,
   g_tls_client_connection_set_validation_flags (G_TLS_CLIENT_CONNECTION (test->client_connection),
                                                 G_TLS_CERTIFICATE_VALIDATE_ALL & ~G_TLS_CERTIFICATE_EXPIRED);
 
-#ifdef BACKEND_IS_GNUTLS
-  g_set_error_literal (&test->expected_server_error, G_TLS_ERROR, G_TLS_ERROR_NOT_TLS, "");
-#endif
-
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   g_assert_error (test->read_error, G_TLS_ERROR, G_TLS_ERROR_BAD_CERTIFICATE);
 
-#ifdef BACKEND_IS_OPENSSL
+#ifdef BACKEND_IS_GNUTLS
+  g_assert_error (test->server_error, G_TLS_ERROR, G_TLS_ERROR_NOT_TLS);
+#elif BACKEND_IS_OPENSSL
+  /* FIXME: This is not OK. There should be an error here. */
   g_assert_no_error (test->server_error);
 #endif
 }
@@ -994,6 +975,7 @@ test_client_auth_connection (TestConnection *test,
 
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   g_assert_no_error (test->read_error);
   g_assert_no_error (test->server_error);
@@ -1028,6 +1010,7 @@ test_client_auth_connection (TestConnection *test,
 
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   g_assert_no_error (test->read_error);
   g_assert_no_error (test->server_error);
@@ -1051,54 +1034,6 @@ test_client_auth_rehandshake (TestConnection *test,
   test_client_auth_connection (test, data);
 }
 
-/* In TLS 1.3 the client handshake succeeds before the client has sent
- * its certificate to the server, so the client doesn't realize the
- * server has rejected its certificate until it tries performing I/O.
- * This results in different errors bubbling up to the API level. The
- * differences are unfortunate but difficult to avoid.
- *
- * FIXME: This isn't good to have different API behavior depending on
- * the version of GnuTLS in use. And how is OpenSSL supposed to deal
- * with this?
- */
-static gboolean
-client_can_receive_certificate_required_errors (TestConnection *test)
-{
-#ifdef BACKEND_IS_GNUTLS
-  gnutls_priority_t priority_cache;
-  int ret;
-  int i;
-  int nprotos;
-  static int max_proto = 0;
-  const guint *protos;
-
-  /* Determine whether GNUTLS_TLS1_3 is available at *runtime* (using
-   * the default priority) so that these tests work in Fedora 28, which
-   * has GnuTLS 3.6 (and therefore GNUTLS_TLS1_3) but with TLS 1.3
-   * disabled.
-   */
-  if (max_proto == 0)
-    {
-      ret = gnutls_priority_init (&priority_cache, "NORMAL", NULL);
-      g_assert_cmpint (ret, ==, GNUTLS_E_SUCCESS);
-
-      nprotos = gnutls_priority_protocol_list (priority_cache, &protos);
-
-      for (i = 0; i < nprotos && protos[i] <= GNUTLS_TLS_VERSION_MAX; i++)
-        {
-          if (protos[i] > max_proto)
-            max_proto = protos[i];
-        }
-
-      gnutls_priority_deinit (priority_cache);
-    }
-
-  return max_proto <= GNUTLS_TLS1_2;
-#else
-  return TRUE;
-#endif
-}
-
 static void
 test_client_auth_failure (TestConnection *test,
                           gconstpointer   data)
@@ -1133,30 +1068,19 @@ test_client_auth_failure (TestConnection *test,
   g_signal_connect (test->client_connection, "notify::accepted-cas",
                     G_CALLBACK (on_notify_accepted_cas), &accepted_changed);
 
-  if (!client_can_receive_certificate_required_errors (test))
-    g_set_error_literal (&test->expected_client_close_error, G_IO_ERROR, G_IO_ERROR_BROKEN_PIPE, "");
-  g_set_error_literal (&test->expected_server_error, G_TLS_ERROR, G_TLS_ERROR_CERTIFICATE_REQUIRED, "");
-
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
-  /* In TLS 1.2 we'll notice that a server cert was requested. For TLS 1.3 we
-   * just get dropped, usually G_TLS_ERROR_MISC but possibly also broken pipe.
-   */
-  if (client_can_receive_certificate_required_errors (test))
-    g_assert_error (test->read_error, G_TLS_ERROR, G_TLS_ERROR_CERTIFICATE_REQUIRED);
-  else if (!g_error_matches (test->read_error, G_IO_ERROR, G_IO_ERROR_BROKEN_PIPE))
-    g_assert_error (test->read_error, G_TLS_ERROR, G_TLS_ERROR_MISC);
+  g_assert_error (test->read_error, G_TLS_ERROR, G_TLS_ERROR_CERTIFICATE_REQUIRED);
   g_assert_error (test->server_error, G_TLS_ERROR, G_TLS_ERROR_CERTIFICATE_REQUIRED);
 
   g_assert_true (accepted_changed);
 
   g_object_unref (test->client_connection);
   g_clear_object (&test->server_connection);
-  g_clear_error (&test->expected_client_close_error);
   g_clear_error (&test->read_error);
   g_clear_error (&test->server_error);
-  g_clear_error (&test->expected_server_error);
 
   /* Now start a new connection to the same server with a valid client cert;
    * this should succeed, and not use the cached failed session from above */
@@ -1189,6 +1113,7 @@ test_client_auth_failure (TestConnection *test,
 
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   g_assert_no_error (test->read_error);
   g_assert_no_error (test->server_error);
@@ -1210,7 +1135,7 @@ test_client_auth_fail_missing_client_private_key (TestConnection *test,
   GError *error = NULL;
 
 #ifdef BACKEND_IS_OPENSSL
-  g_test_skip("this new test does not work with openssl, more research needed");
+  g_test_skip ("this new test does not work with openssl, more research needed");
   return;
 #endif
 
@@ -1240,12 +1165,12 @@ test_client_auth_fail_missing_client_private_key (TestConnection *test,
   g_tls_client_connection_set_validation_flags (G_TLS_CLIENT_CONNECTION (test->client_connection),
                                                 G_TLS_CERTIFICATE_VALIDATE_ALL);
 
-  g_set_error_literal (&test->expected_server_error, G_TLS_ERROR, G_TLS_ERROR_NOT_TLS, "");
-
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   g_assert_error (test->read_error, G_TLS_ERROR, G_TLS_ERROR_CERTIFICATE_REQUIRED);
+  g_assert_error (test->server_error, G_TLS_ERROR, G_TLS_ERROR_NOT_TLS);
 }
 
 static void
@@ -1288,6 +1213,7 @@ test_client_auth_request_cert (TestConnection *test,
 
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   g_assert_no_error (test->read_error);
   g_assert_no_error (test->server_error);
@@ -1329,22 +1255,15 @@ test_client_auth_request_fail (TestConnection *test,
   g_tls_client_connection_set_validation_flags (G_TLS_CLIENT_CONNECTION (test->client_connection),
                                                 G_TLS_CERTIFICATE_VALIDATE_ALL);
 
-  if (!client_can_receive_certificate_required_errors (test))
-    g_set_error_literal (&test->expected_client_close_error, G_IO_ERROR, G_IO_ERROR_BROKEN_PIPE, "");
-  g_set_error_literal (&test->expected_server_error, G_TLS_ERROR, G_TLS_ERROR_CERTIFICATE_REQUIRED, "");
-
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
-  /* FIXME: G_FILE_ERROR_ACCES is not a very great error to get here. */
-  if (client_can_receive_certificate_required_errors (test))
-#if OPENSSL_VERSION_NUMBER < 0x10101000L || defined (LIBRESSL_VERSION_NUMBER)
-    g_assert_error (test->read_error, G_FILE_ERROR, G_FILE_ERROR_ACCES);
-#else
-    g_assert_error (test->read_error, G_TLS_ERROR, G_TLS_ERROR_CERTIFICATE_REQUIRED);
-#endif
-  else if (!g_error_matches (test->read_error, G_IO_ERROR, G_IO_ERROR_BROKEN_PIPE))
-    g_assert_error (test->read_error, G_TLS_ERROR, G_TLS_ERROR_MISC);
+  /* G_FILE_ERROR_ACCES is the error returned by our mock interaction object
+   * when the GTlsInteraction's certificate request fails.
+   */
+  g_assert_error (test->read_error, G_FILE_ERROR, G_FILE_ERROR_ACCES);
+  g_assert_error (test->server_error, G_TLS_ERROR, G_TLS_ERROR_CERTIFICATE_REQUIRED);
 
   g_io_stream_close (test->server_connection, NULL, NULL);
   g_io_stream_close (test->client_connection, NULL, NULL);
@@ -1376,6 +1295,7 @@ test_connection_no_database (TestConnection *test,
 
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   g_assert_no_error (test->read_error);
   g_assert_no_error (test->server_error);
@@ -1413,10 +1333,6 @@ test_failed_connection (TestConnection *test,
   g_assert_no_error (error);
   g_object_unref (connection);
 
-#ifdef BACKEND_IS_GNUTLS
-  g_set_error_literal (&test->expected_server_error, G_TLS_ERROR, G_TLS_ERROR_NOT_TLS, "");
-#endif
-
   g_tls_connection_handshake_async (G_TLS_CONNECTION (test->client_connection),
                                     G_PRIORITY_DEFAULT, NULL,
                                     handshake_failed_cb, test);
@@ -1427,10 +1343,14 @@ test_failed_connection (TestConnection *test,
 
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   g_assert_error (test->read_error, G_TLS_ERROR, G_TLS_ERROR_BAD_CERTIFICATE);
 
-#ifdef BACKEND_IS_OPENSSL
+#if BACKEND_IS_GNUTLS
+  g_assert_error (test->server_error, G_TLS_ERROR, G_TLS_ERROR_NOT_TLS);
+#elif BACKEND_IS_OPENSSL
+  /* FIXME: This is not OK. There should be an error here. */
   g_assert_no_error (test->server_error);
 #endif
 }
@@ -1473,6 +1393,7 @@ test_connection_socket_client (TestConnection *test,
   g_socket_client_connect_async (client, G_SOCKET_CONNECTABLE (test->address),
                                  NULL, socket_client_connected, test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   connection = (GSocketConnection *)test->client_connection;
   test->client_connection = NULL;
@@ -1515,35 +1436,30 @@ test_connection_socket_client_failed (TestConnection *test,
   g_socket_client_set_tls (client, TRUE);
   /* this time we don't adjust the validation flags */
 
-#ifdef BACKEND_IS_GNUTLS
-  g_set_error_literal (&test->expected_server_error, G_TLS_ERROR, G_TLS_ERROR_NOT_TLS, "");
-#endif
-
   g_socket_client_connect_async (client, G_SOCKET_CONNECTABLE (test->address),
                                  NULL, socket_client_failed, test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
+
+#ifdef BACKEND_IS_GNUTLS
+  g_assert_error (test->server_error, G_TLS_ERROR, G_TLS_ERROR_NOT_TLS);
+#else
+  /* FIXME: This is not OK. There should be an error here. */
+#endif
 
   g_object_unref (client);
 }
 
-static void
-socket_client_timed_out_write (GObject      *source,
-                               GAsyncResult *result,
-                               gpointer      user_data)
+static gboolean
+socket_client_timed_out_write (gpointer user_data)
 {
   TestConnection *test = user_data;
-  GSocketConnection *connection;
   GInputStream *input_stream;
   GOutputStream *output_stream;
   GError *error = NULL;
   gchar buffer[TEST_DATA_LENGTH];
   gssize size;
 
-  connection = g_socket_client_connect_finish (G_SOCKET_CLIENT (source),
-                                               result, &error);
-  g_assert_no_error (error);
-  test->client_connection = G_IO_STREAM (connection);
-
   input_stream = g_io_stream_get_input_stream (test->client_connection);
   output_stream = g_io_stream_get_output_stream (test->client_connection);
 
@@ -1567,6 +1483,34 @@ socket_client_timed_out_write (GObject      *source,
   g_assert_cmpint (size, ==, TEST_DATA_LENGTH);
 
   g_main_loop_quit (test->loop);
+
+  return G_SOURCE_REMOVE;
+}
+
+static void
+socket_client_timed_out_write_connected (GObject      *source,
+                                         GAsyncResult *result,
+                                         gpointer      user_data)
+{
+  TestConnection *test = user_data;
+  GSocketConnection *connection;
+  GError *error = NULL;
+
+  connection = g_socket_client_connect_finish (G_SOCKET_CLIENT (source),
+                                               result, &error);
+  g_assert_no_error (error);
+  test->client_connection = G_IO_STREAM (connection);
+
+  /* We need to use an idle callback here to guarantee that the upcoming call
+   * to g_input_stream_read() executes on the next iteration of the main
+   * context. Otherwise, we could deadlock ourselves: the read would not be able
+   * to complete if GTask executes socket_client_timed_out_write_connected()
+   * using g_task_return_now() instead of posting the invocation to the next
+   * iteration of the main context, because the server will not progress until
+   * the main context is iterated, but iteration would be blocked waiting for
+   * client's read to complete.
+   */
+  g_idle_add (socket_client_timed_out_write, test);
 }
 
 static void
@@ -1591,7 +1535,7 @@ test_connection_read_time_out_write (TestConnection *test,
   g_socket_client_set_tls_validation_flags (client, flags);
 
   g_socket_client_connect_async (client, G_SOCKET_CONNECTABLE (test->address),
-                                 NULL, socket_client_timed_out_write, test);
+                                 NULL, socket_client_timed_out_write_connected, test);
 
   g_main_loop_run (test->loop);
 
@@ -1661,7 +1605,7 @@ simul_async_write_complete (GObject      *object,
   if (test->nwrote < TEST_DATA_LENGTH)
     {
       g_output_stream_write_async (G_OUTPUT_STREAM (object),
-                                   TEST_DATA + test->nwrote,
+                                   &TEST_DATA[test->nwrote],
                                    TEST_DATA_LENGTH - test->nwrote,
                                    G_PRIORITY_DEFAULT, NULL,
                                    simul_async_write_complete, test);
@@ -1699,6 +1643,7 @@ test_simultaneous_async (TestConnection *test,
                                simul_async_write_complete, test);
 
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   g_assert_cmpint (test->nread, ==, TEST_DATA_LENGTH);
   g_assert_cmpint (test->nwrote, ==, TEST_DATA_LENGTH);
@@ -1751,7 +1696,7 @@ simul_write_thread (gpointer user_data)
   while (test->nwrote < TEST_DATA_LENGTH)
     {
       nwrote = g_output_stream_write (ostream,
-                                      TEST_DATA + test->nwrote,
+                                      &TEST_DATA[test->nwrote],
                                       MIN (TEST_DATA_LENGTH / 2, TEST_DATA_LENGTH - test->nwrote),
                                       NULL, &error);
       g_assert_no_error (error);
@@ -1905,8 +1850,10 @@ test_async_implicit_handshake (TestConnection *test, gconstpointer   data)
                          test, NULL);
 
   g_source_attach (input_source, NULL);
+  g_source_unref (input_source);
 
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   g_io_stream_close (G_IO_STREAM (test->client_connection), NULL, &error);
   g_assert_no_error (error);
@@ -1961,18 +1908,14 @@ test_fallback (TestConnection *test,
 #pragma GCC diagnostic pop
 #endif
 
-#if GLIB_CHECK_VERSION(2, 60, 0)
-  g_set_error_literal (&test->expected_server_error, G_TLS_ERROR, G_TLS_ERROR_INAPPROPRIATE_FALLBACK, "");
-#else
-  g_set_error_literal (&test->expected_server_error, G_TLS_ERROR, G_TLS_ERROR_MISC, "");
-#endif
-
   g_tls_connection_handshake_async (tlsconn, G_PRIORITY_DEFAULT, NULL,
                                     quit_on_handshake_complete, test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   /* The server should detect a protocol downgrade attack and terminate the connection.
    */
+  g_assert_error (test->server_error, G_TLS_ERROR, G_TLS_ERROR_INAPPROPRIATE_FALLBACK);
 
   g_io_stream_close (test->client_connection, NULL, &error);
   g_assert_no_error (error);
@@ -1999,6 +1942,17 @@ test_output_stream_close (TestConnection *test,
   gboolean handshake_complete = FALSE;
   gssize size;
 
+#ifdef BACKEND_IS_OPENSSL
+# if OPENSSL_VERSION_NUMBER >= 0x10101000L
+  /* FIXME: This test fails most of the times with openssl 1.1.1, my guess is that
+   * there is still some threading issue and we endup calling input_stream_read
+   * from different threads and the same time.
+   */
+  g_test_skip ("this is not supported with openssl 1.1.1");
+  return;
+# endif
+#endif
+
   connection = start_async_server_and_connect_to_it (test, G_TLS_AUTHENTICATION_NONE);
   test->client_connection = g_tls_client_connection_new (connection, test->identity, &error);
   g_assert_no_error (error);
@@ -2037,6 +1991,7 @@ test_output_stream_close (TestConnection *test,
    */
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   g_assert_no_error (test->read_error);
   g_assert_no_error (test->server_error);
@@ -2074,15 +2029,15 @@ test_garbage_database (TestConnection *test,
   g_tls_client_connection_set_validation_flags (G_TLS_CLIENT_CONNECTION (test->client_connection),
                                                 G_TLS_CERTIFICATE_VALIDATE_ALL);
 
-  g_set_error_literal (&test->expected_server_error, G_TLS_ERROR, G_TLS_ERROR_NOT_TLS, "");
-
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   /* Should reject the server's certificate, because our TLS database contains
    * no valid certificates.
    */
   g_assert_error (test->read_error, G_TLS_ERROR, G_TLS_ERROR_BAD_CERTIFICATE);
+  g_assert_error (test->server_error, G_TLS_ERROR, G_TLS_ERROR_NOT_TLS);
 }
 
 static void
@@ -2140,7 +2095,6 @@ test_alpn (TestConnection *test,
   return;
 #endif
 
-#if GLIB_CHECK_VERSION(2, 60, 0)
   GIOStream *connection;
   GError *error = NULL;
 
@@ -2165,15 +2119,13 @@ test_alpn (TestConnection *test,
 
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   g_assert_no_error (test->read_error);
   g_assert_no_error (test->server_error);
 
   g_assert_cmpstr (g_tls_connection_get_negotiated_protocol (G_TLS_CONNECTION (test->server_connection)), ==, negotiated_protocol);
   g_assert_cmpstr (g_tls_connection_get_negotiated_protocol (G_TLS_CONNECTION (test->client_connection)), ==, negotiated_protocol);
-#else
-  g_test_skip ("no support for ALPN in this GLib version");
-#endif
 }
 
 static void
@@ -2272,6 +2224,7 @@ test_sync_op_during_handshake (TestConnection *test,
 
   read_test_data_async (test);
   g_main_loop_run (test->loop);
+  wait_until_server_finished (test);
 
   g_assert_no_error (test->read_error);
   g_assert_no_error (test->server_error);
@@ -2290,73 +2243,73 @@ main (int   argc,
   g_setenv ("GIO_USE_TLS", BACKEND, TRUE);
   g_assert (g_ascii_strcasecmp (G_OBJECT_TYPE_NAME (g_tls_backend_get_default ()), "GTlsBackend" BACKEND) == 0);
 
-  g_test_add ("/tls/connection/basic", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/basic", TestConnection, NULL,
               setup_connection, test_basic_connection, teardown_connection);
-  g_test_add ("/tls/connection/verified", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/verified", TestConnection, NULL,
               setup_connection, test_verified_connection, teardown_connection);
-  g_test_add ("/tls/connection/verified-chain", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/verified-chain", TestConnection, NULL,
               setup_connection, test_verified_chain, teardown_connection);
-  g_test_add ("/tls/connection/verified-chain-with-redundant-root-cert", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/verified-chain-with-redundant-root-cert", TestConnection, NULL,
               setup_connection, test_verified_chain_with_redundant_root_cert, teardown_connection);
-  g_test_add ("/tls/connection/verified-chain-with-duplicate-server-cert", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/verified-chain-with-duplicate-server-cert", TestConnection, NULL,
               setup_connection, test_verified_chain_with_duplicate_server_cert, teardown_connection);
-  g_test_add ("/tls/connection/verified-unordered-chain", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/verified-unordered-chain", TestConnection, NULL,
               setup_connection, test_verified_unordered_chain, teardown_connection);
-  g_test_add ("/tls/connection/verified-chain-with-alternative-ca-cert", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/verified-chain-with-alternative-ca-cert", TestConnection, NULL,
               setup_connection, test_verified_chain_with_alternative_ca_cert, teardown_connection);
-  g_test_add ("/tls/connection/invalid-chain-with-alternative-ca-cert", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/invalid-chain-with-alternative-ca-cert", TestConnection, NULL,
               setup_connection, test_invalid_chain_with_alternative_ca_cert, teardown_connection);
-  g_test_add ("/tls/connection/client-auth", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/client-auth", TestConnection, NULL,
               setup_connection, test_client_auth_connection, teardown_connection);
-  g_test_add ("/tls/connection/client-auth-rehandshake", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/client-auth-rehandshake", TestConnection, NULL,
               setup_connection, test_client_auth_rehandshake, teardown_connection);
-  g_test_add ("/tls/connection/client-auth-failure", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/client-auth-failure", TestConnection, NULL,
               setup_connection, test_client_auth_failure, teardown_connection);
-  g_test_add ("/tls/connection/client-auth-fail-missing-client-private-key", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/client-auth-fail-missing-client-private-key", TestConnection, NULL,
               setup_connection, test_client_auth_fail_missing_client_private_key, teardown_connection);
-  g_test_add ("/tls/connection/client-auth-request-cert", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/client-auth-request-cert", TestConnection, NULL,
               setup_connection, test_client_auth_request_cert, teardown_connection);
-  g_test_add ("/tls/connection/client-auth-request-fail", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/client-auth-request-fail", TestConnection, NULL,
               setup_connection, test_client_auth_request_fail, teardown_connection);
-  g_test_add ("/tls/connection/no-database", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/no-database", TestConnection, NULL,
               setup_connection, test_connection_no_database, teardown_connection);
-  g_test_add ("/tls/connection/failed", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/failed", TestConnection, NULL,
               setup_connection, test_failed_connection, teardown_connection);
-  g_test_add ("/tls/connection/socket-client", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/socket-client", TestConnection, NULL,
               setup_connection, test_connection_socket_client, teardown_connection);
-  g_test_add ("/tls/connection/socket-client-failed", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/socket-client-failed", TestConnection, NULL,
               setup_connection, test_connection_socket_client_failed, teardown_connection);
-  g_test_add ("/tls/connection/read-time-out-then-write", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/read-time-out-then-write", TestConnection, NULL,
               setup_connection, test_connection_read_time_out_write, teardown_connection);
-  g_test_add ("/tls/connection/simultaneous-async", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/simultaneous-async", TestConnection, NULL,
               setup_connection, test_simultaneous_async, teardown_connection);
-  g_test_add ("/tls/connection/simultaneous-sync", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/simultaneous-sync", TestConnection, NULL,
               setup_connection, test_simultaneous_sync, teardown_connection);
-  g_test_add ("/tls/connection/simultaneous-async-rehandshake", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/simultaneous-async-rehandshake", TestConnection, NULL,
               setup_connection, test_simultaneous_async_rehandshake, teardown_connection);
-  g_test_add ("/tls/connection/simultaneous-sync-rehandshake", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/simultaneous-sync-rehandshake", TestConnection, NULL,
               setup_connection, test_simultaneous_sync_rehandshake, teardown_connection);
-  g_test_add ("/tls/connection/close-immediately", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/close-immediately", TestConnection, NULL,
               setup_connection, test_close_immediately, teardown_connection);
-  g_test_add ("/tls/connection/async-implicit-handshake", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/async-implicit-handshake", TestConnection, NULL,
               setup_connection, test_async_implicit_handshake, teardown_connection);
-  g_test_add ("/tls/connection/output-stream-close", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/output-stream-close", TestConnection, NULL,
               setup_connection, test_output_stream_close, teardown_connection);
-  g_test_add ("/tls/connection/fallback", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/fallback", TestConnection, NULL,
               setup_connection, test_fallback, teardown_connection);
-  g_test_add ("/tls/connection/garbage-database", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/garbage-database", TestConnection, NULL,
               setup_connection, test_garbage_database, teardown_connection);
-  g_test_add ("/tls/connection/readwrite-after-connection-destroyed", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/readwrite-after-connection-destroyed", TestConnection, NULL,
               setup_connection, test_readwrite_after_connection_destroyed, teardown_connection);
-  g_test_add ("/tls/connection/alpn/match", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/alpn/match", TestConnection, NULL,
               setup_connection, test_alpn_match, teardown_connection);
-  g_test_add ("/tls/connection/alpn/no-match", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/alpn/no-match", TestConnection, NULL,
               setup_connection, test_alpn_no_match, teardown_connection);
-  g_test_add ("/tls/connection/alpn/client-only", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/alpn/client-only", TestConnection, NULL,
               setup_connection, test_alpn_client_only, teardown_connection);
-  g_test_add ("/tls/connection/alpn/server-only", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/alpn/server-only", TestConnection, NULL,
               setup_connection, test_alpn_server_only, teardown_connection);
-  g_test_add ("/tls/connection/sync-op-during-handshake", TestConnection, NULL,
+  g_test_add ("/tls/" BACKEND "/connection/sync-op-during-handshake", TestConnection, NULL,
               setup_connection, test_sync_op_during_handshake, teardown_connection);
 
   ret = g_test_run ();
